from abc import ABC, abstractmethod
from typing import Any, Dict, List, Optional, Set, Union, Tuple, Type
from enum import Enum
import inspect
import re

class Rule(ABC):
    """Abstract base class for all validation rules"""
    
    _rule_classes: Dict[str, Type['Rule']] = {}
    
    def __init__(self, *params: str):
        self._validator: Optional['Validator'] = None
        self._params: List[str] = list(params)
        
    def __init_subclass__(cls):
        cls.rule_name = cls.pascal_to_snake(cls.__name__) if not hasattr(cls, '_name') else cls._name
        cls._register_rule_class()
        cls._generate_rule_methods()
        
    @property
    def params(self):
        params = []
        for rule_set in self._params:
            if isinstance(rule_set, (list, tuple)):
                params.append(tuple(rule_set))
            else:
                params.append(rule_set)
        return tuple(params)
    
    @classmethod
    def _register_rule_class(cls):
        cls._rule_classes[cls.rule_name] = cls
    
    @classmethod
    def _generate_rule_methods(cls):
        for rule_name, rule_class in cls._rule_classes.items():
            @classmethod
            def method(cls, *params, rule_class=rule_class):
                return rule_class(*params)
            
            setattr(cls.__class__, rule_name, method)
            
    @classmethod
    def _require_parameter_count(self, count, params, rule):
        if getattr(rule, '_accept_closure', False) and (isinstance(params[0], bool) or callable(params[0])):
            return
        
        if count > len(params):
            raise ValueError(f"Validation rule {rule.rule_name} requires at least {count} parameter")
            
    @params.setter
    def params(self, value: List[str]) -> None:
        self._params = value
    
    def set_validator(self, validator: 'Validator') -> None:
        """Set the validator instance this rule belongs to."""
        self._validator = validator
        
    def set_field_exists(self, exists: bool):
        self._field_exists = exists
    
    @property
    def validator(self) -> 'Validator':
        """Get the validator instance."""
        if self._validator is None:
            raise RuntimeError("Validator not set for this rule!")
        return self._validator
    
    @property
    def field_exists(self):
        return self._field_exists
    
    def get_field_value(self, field_name, default=''):
        return str(self.validator.data.get(field_name, default))
    
    @staticmethod
    def is_empty(value):
        return value in (None, '', [], {})
    
    @abstractmethod
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        """Validate a field's value."""
        pass
    
    @abstractmethod
    def message(self, field: str) -> str:
        """Generate an error message if validation fails."""
        pass
    
    def replacements(self, field, value) -> Dict[str, str]:
        """Default replacements for all rules"""
        return {':attribute': self._get_display_name(field), 'value': value }
    
    def _get_display_name(self, fields: str) -> str:
        single_input = isinstance(fields, str)
        if single_input:
            fields = [fields]
        
        attributes = []
        
        for field in fields:
            if field in self.validator.custom_attributes:
                attributes.append(self.validator.custom_attributes[field])
                continue
                
            parts = field.split('.')
            base_name = parts[-1].replace('_', ' ').title()
            
            for i in range(len(parts), 0, -1):
                wildcard = '.'.join(parts[:i]) + '.*'
                if wildcard in self.validator.custom_attributes:
                    attributes.append(self.validator.custom_attributes[wildcard])
                    break
            else:
                attributes.append(self.validator.custom_attributes.get(parts[-1], base_name))
        
        return attributes[0] if single_input else attributes
    
    @property
    @abstractmethod
    def rule_name(self) -> str:
        """Return the name of the rule for error messages."""
        pass
    
    def _parse_option_values(self, field: str, params: List[str], raise_for_error=True) -> List[Any]:
        """Parse parameters into allowed values, supporting both Enum class and literal values"""
        
        if not params and raise_for_error:
            raise ValueError(
                f"{self.rule_name} rule requires parameters. "
                f"Use '({self.rule_name}, EnumClass)' or '{self.rule_name}:val1,val2'"
            )
            
        enum_params = [param for param in params if inspect.isclass(param) and issubclass(param, Enum)]
        params = [param for param in params if param not in enum_params]
        
        for enum_param in enum_params:
            params.extend([e.value for e in enum_param])
            
        params = set([str(param) for param in params])
            
        param_str = ' ,'.join(params)
        
        return [v.strip() for v in param_str.split(',') if v.strip()]
    
    def pascal_to_snake(name):
        """Convert PascalCase to snake_case"""
        # Handle kasus khusus terlebih dahulu
        special_cases = {
            'UUIDRule': 'uuid',
            'IPRule': 'ip',
            'URLRule': 'url'
        }
        if name in special_cases:
            return special_cases[name]
        
        # Konversi regular PascalCase ke snake_case
        s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
        result = re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()
        return result.replace('_rule', '')


from .base import Rule
from typing import Any, Dict, List, Optional, Set, Union, Tuple, Type
import mimetypes

class FileRule(Rule):
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        # 1. Cek file object framework (Flask/Werkzeug/FastAPI)
        if (hasattr(value, 'filename') and 
            hasattr(value, 'stream') and 
            hasattr(value, 'content_type')):
            return True
            
        # 2. Cek file-like object umum
        if hasattr(value, 'read') and callable(value.read):
            return True
            
        # 3. Cek path file yang valid (string)
        if isinstance(value, str):
            return (
                '.' in value and                  # Harus punya extension
                not value.startswith('data:') and # Bukan data URI
                not value.strip().startswith('<') # Bukan XML/HTML
            )
            
        # 4. Cek binary data langsung
        if isinstance(value, (bytes, bytearray)):
            return len(value) > 0  # Pastikan tidak kosong
            
        return False
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must be a valid file"

class DimensionsRule(Rule):
    def __init__(self, *params):
        super().__init__(*params)
        self._count_parameter = 1
        self._constraints = {
            'min_width': 0,
            'min_height': 0,
            'max_width': float('inf'),
            'max_height': float('inf'),
            'width': None,
            'height': None,
            'ratio': None
        }

    def maxWidth(self, value: int) -> 'DimensionsRule':
        self._constraints['width'] = None
        self._constraints['max_width'] = value
        return self

    def maxHeight(self, value: int) -> 'DimensionsRule':
        self._constraints['height'] = None
        self._constraints['max_height'] = value
        return self

    def ratio(self, value: float) -> 'DimensionsRule':
        self._constraints['ratio'] = value
        return self

    def minWidth(self, value: int) -> 'DimensionsRule':
        self._constraints['width'] = None
        self._constraints['min_width'] = value
        return self

    def minHeight(self, value: int) -> 'DimensionsRule':
        self._constraints['height'] = None
        self._constraints['min_height'] = value
        return self
    
    def _validate_constraint_logic(self):
        if self._constraints['width'] is not None:
            if (self._constraints['min_width'] > 0 or 
                self._constraints['max_width'] < float('inf')):
                raise ValueError("Cannot specify both exact width and min/max width")

        if self._constraints['max_width'] < self._constraints['min_width']:
            raise ValueError(f"max_width ({self._constraints['max_width']}) "
                f"cannot be less than min_width ({self._constraints['min_width']})")

        if self._constraints['height'] is not None:
            if (self._constraints['min_height'] > 0 or 
                self._constraints['max_height'] < float('inf')):
                raise ValueError("Cannot specify both exact height and min/max height")

        if self._constraints['max_height'] < self._constraints['min_height']:
            raise ValueError(f"max_height ({self._constraints['max_height']}) "
                f"cannot be less than min_height ({self._constraints['min_height']})")
        
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        self._validate_constraint_logic()
        try:
            # 2. Parse parameters
            for param in params:
                # Format Laravel: min_width=100
                if '=' in param:
                    key, val = param.split('=', 1)
                    if key in self._constraints:
                        try:
                            self._constraints[key] = float(val)
                        except ValueError:
                            continue
                
                # Format legacy: 100x200
                elif 'x' in param:
                    try:
                        width, height = map(int, param.split('x'))
                        self._constraints['width'] = width
                        self._constraints['height'] = height
                    except ValueError:
                        continue
                
                # Format ratio: 3/2
                elif '/' in param:
                    try:
                        self._constraints['ratio'] = param
                    except ValueError:
                        continue

            # 3. Load image
            img = self._load_image(value)
            if not img:
                return False

            width, height = img.size
            actual_ratio = round(width / height, 2)

            # 4. Validate _constraints
            checks = [
                (self._constraints['width'] is None or width == self._constraints['width']),
                (self._constraints['height'] is None or height == self._constraints['height']),
                width >= self._constraints['min_width'],
                height >= self._constraints['min_height'],
                width <= self._constraints['max_width'],
                height <= self._constraints['max_height'],
                self._check_ratio(self._constraints['ratio'], actual_ratio)
            ]
            
            return all(checks)

        except Exception as e:
            print(f"Dimension validation error: {str(e)}")
            return False

    def _load_image(self, value):
        """Helper to load image from different sources"""
        try:
            from PIL import Image
            import io
            
            if hasattr(value, 'read'):  # File-like object
                value.seek(0)
                img = Image.open(value)
                value.seek(0)
                return img
            elif isinstance(value, bytes):  # Bytes
                return Image.open(io.BytesIO(value))
            elif isinstance(value, str):  # File path
                if value.lower().endswith(('.jpg', '.jpeg', '.png', '.gif')):
                    return Image.open(value)
            return None
        except:
            return None

    def _check_ratio(self, ratio_constraint, actual_ratio):
        """Validate aspect ratio"""
        if not ratio_constraint:
            return True
            
        try:
            numerator, denominator = map(float, ratio_constraint.split('/'))
            expected_ratio = round(numerator / denominator, 2)
            return actual_ratio == expected_ratio
        except:
            return False

    def message(self, field: str, params: List[str]) -> str:
        constraints = []
        
        if self._constraints['width'] is not None:
            constraints.append(f"width={self._constraints['width']}")
        if self._constraints['height'] is not None:
            constraints.append(f"height={self._constraints['height']}")
        if self._constraints['min_width'] > 0:
            constraints.append(f"min_width={self._constraints['min_width']}")
        if self._constraints['max_width'] < float('inf'):
            constraints.append(f"max_width={self._constraints['max_width']}")
        if self._constraints['min_height'] > 0:
            constraints.append(f"min_height={self._constraints['min_height']}")
        if self._constraints['max_height'] < float('inf'):
            constraints.append(f"max_height={self._constraints['max_height']}")
        if self._constraints['ratio'] is not None:
            constraints.append(f"ratio={self._constraints['ratio']:.2f}")
        
        return f"The :attribute image must satisfy: {', '.join(constraints)}"
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':width': self._constraints['width'],
            ':min_width': self._constraints['min_width'],
            ':max_width': self._constraints['max_width'],
            ':height': self._constraints['height'],
            ':min_height': self._constraints['min_height'],
            ':max_height': self._constraints['max_height'],
            ':ratio': self._constraints['ratio'],
        }
        
        return replacements
    
class ExtensionsRule(Rule):
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        filename = (
            value.filename if hasattr(value, 'filename') 
            else str(value)
        ).lower()
        
        if '.' not in filename:
            return False
            
        ext = filename.rsplit('.', 1)[1]
        return ext in [e.lower().strip() for e in params]
    
    def message(self, field: str, params: List[str]) -> str:
        return f"File :attribute must have one of these extensions: :values"
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':values': ', '.join(self._params)
        }
        
        return replacements
    
class ImageRule(Rule):
    VALID_EXTENSIONS = {'jpg', 'jpeg', 'png', 'gif', 'bmp', 'webp'}
    VALID_MIME_TYPES = {
        'image/jpeg', 'image/png', 
        'image/gif', 'image/bmp', 'image/webp'
    }

    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        # Check basic file attributes
        if not hasattr(value, 'filename') and not isinstance(value, (str, bytes)):
            return False
            
        # Check extension if available
        if hasattr(value, 'filename'):
            ext = value.filename.rsplit('.', 1)[-1].lower()
            if ext not in self.VALID_EXTENSIONS:
                return False
                
        # Check MIME type if available
        if hasattr(value, 'content_type'):
            if value.content_type not in self.VALID_MIME_TYPES:
                return False
                
        return True
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must be a valid image file (JPEG, PNG, GIF, BMP, or WebP)"
    
class MimeTypesRule(Rule):
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        # Check from content_type attribute
        if hasattr(value, 'content_type'):
            return value.content_type in params
            
        # Check from mimetype attribute
        if hasattr(value, 'mimetype'):
            return value.mimetype in params
            
        return False
    
    def message(self, field: str, params: List[str]) -> str:
        return f"File :attribute must be one of these types: :values"
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':values': ', '.join(self._params)
        }
        
        return replacements
    
class MimeTypeByExtensionRule(Rule):
    _name = 'mimes'
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if value is None or not hasattr(value, 'filename'):
            return False
            
        mimetypes.init()
        
        extension = value.filename.split('.')[-1].lower()
        
        mime_type = mimetypes.guess_type(f"file.{extension}")[0]
        
        return mime_type in params if mime_type else False
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must be one of these types: :values"
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':values': ', '.join(self._params)
        }
        
        return replacements


from .base import Rule
from typing import Any, Dict, List, Optional, Set, Union, Tuple, Type
import re
import ipaddress
import json
import uuid
import dns.resolver
import idna

# =============================================
# FORMAT VALIDATION RULES
# =============================================

class EmailRule(Rule):
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        """
        Available params:
        - rfc: RFCValidation (default)
        - strict: NoRFCWarningsValidation
        - dns: DNSCheckValidation
        - spoof: SpoofCheckValidation
        - filter: FilterEmailValidation
        - filter_unicode: FilterEmailValidation::unicode()
        """
        
        if not isinstance(value, str):
            return False

        # Default to RFC validation if no params specified
        validation_types = params if params else ['rfc']

        # Apply all requested validations
        for validation in validation_types:
            if validation == 'rfc' and not self._validate_rfc(value):
                return False
            elif validation == 'strict' and not self._validate_strict(value):
                return False
            elif validation == 'dns' and not self._validate_dns(value):
                return False
            elif validation == 'spoof' and not self._validate_spoof(value):
                return False
            elif validation == 'filter' and not self._validate_filter(value):
                return False
            elif validation == 'filter_unicode' and not self._validate_filter_unicode(value):
                return False

        return True

    def _validate_rfc(self, email: str) -> bool:
        """Strict RFC-compliant email validation (single-line version)"""
        return bool(re.match(
            r"^(?!(\.|\.\.))[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@(?=.{1,255}$)[a-zA-Z0-9-]+(\.[a-zA-Z0-9-]+)*\.[a-zA-Z]{2,}$", 
            email
        ))

    def _validate_strict(self, email: str) -> bool:
        """Strict RFC validation (no warnings)"""
        if not self._validate_rfc(email):
            return False
        
        # No leading/trailing dots
        if email.startswith('.') or email.endswith('.'):
            return False
            
        # No consecutive dots
        if '..' in email.split('@')[0]:
            return False
            
        return True

    def _validate_dns(self, email: str) -> bool:
        """DNS MX record validation"""
        try:
            domain = email.split('@')[-1]
            return bool(dns.resolver.resolve(domain, 'MX'))
        except (dns.resolver.NoAnswer, dns.resolver.NXDOMAIN, 
            dns.resolver.NoNameservers, dns.exception.DNSException):
            return False

    def _validate_spoof(self, email: str) -> bool:
        """Spoof/homograph detection"""
        try:
            domain_part = email.split('@')[-1]
            ascii_domain = idna.encode(domain_part).decode('ascii')
            
            # Check for homographs
            if domain_part != ascii_domain:
                return False
                
            # Check for deceptive characters
            return not any(ord(char) > 127 for char in email)
        except idna.IDNAError:
            return False

    def _validate_filter(self, email: str) -> bool:
        """PHP filter_var compatible validation"""
        return bool(re.match(
            r"^(?!(\.|\.\.))[a-zA-Z0-9.!#$%&'*+/=?^_`{|}~-]+@[a-zA-Z0-9-]+(?:\.[a-zA-Z0-9-]+)*$",
            email
        ))

    def _validate_filter_unicode(self, email: str) -> bool:
        """PHP filter_var with Unicode support"""
        return bool(re.match(
            r"^(?!(\.|\.\.))[\w.!#$%&'*+/=?^_`{|}~-]+@[\w-]+(?:\.[\w-]+)*$",
            email,
            re.UNICODE
        ))

    def message(self, field: str, params: List[str]) -> str:
        base_msg = "The :attribute must be a valid email address"
        
        if 'strict' in params:
            base_msg += " (strict RFC compliance)"
        if 'dns' in params:
            base_msg += " with valid DNS records"
        if 'spoof' in params:
            base_msg += " without spoofed characters"
        if 'filter' in params:
            base_msg += " (PHP filter compatible)"
        if 'filter_unicode' in params:
            base_msg += " (Unicode allowed)"
            
        return f"{base_msg}."
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':values': ', '.join(self._params if self._params else ['rfc']),
        }
        
        return replacements

class UrlRule(Rule):
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if not isinstance(value, str):
            return False
        return bool(re.match(
            r'^(https?:\/\/)?'  # protocol
            r'([\da-z\.-]+)\.'  # domain
            r'([a-z\.]{2,6})'   # top level domain
            r'([\/\w \.-]*)*\/?$',  # path/query
            value
        ))
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must be a valid URL."

class JsonRule(Rule):
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if not isinstance(value, str):
            return False
        try:
            json.loads(value)
            return True
        except json.JSONDecodeError:
            return False
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must be a valid JSON string."

class UuidRule(Rule):
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if not isinstance(value, str):
            return False
        try:
            uuid.UUID(value)
            return True
        except ValueError:
            return False
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must be a valid UUID."

class UlidRule(Rule):
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if not isinstance(value, str):
            return False
        return bool(re.match(r'^[0-9A-HJKMNP-TV-Z]{26}$', value))
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must be a valid ULID."

class IpRule(Rule):
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if not isinstance(value, str):
            return False
        try:
            ipaddress.ip_address(value)
            return True
        except ValueError:
            return False
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must be a valid IP address."

class HexRule(Rule):
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if not isinstance(value, str):
            return False
        return bool(re.match(r'^#?([a-fA-F0-9]{6}|[a-fA-F0-9]{3})$', value))
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must be a valid hexadecimal color code."



from .base import Rule
from typing import Any, Dict, List, Optional, Set, Union, Tuple, Type
from datetime import datetime
import zoneinfo
from dateutil.parser import parse

class DateRule(Rule):
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        if not isinstance(value, str):
            return False
            
        try:
            parse(value)
            return True
        except ValueError:
            return False
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute is not a valid date."

class DateEqualsRule(Rule):
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        
        try:
            if isinstance(value, str):
                date1 = parse(value)
            elif isinstance(value, datetime):
                date1 = value
            else:
                return False
            
            params = list(params)
            params[0] = self.get_field_value(params[0], params[0])
            
            date2 = parse(params[0])
            return date1 == date2
        except ValueError as e:
            return False
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must be equal to :value."
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':value': self.get_field_value(self._params[0], self._params[0])
        }
        
        return replacements
    
class AfterRule(Rule):
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        try:
            if isinstance(value, str):
                date_value = parse(value)
            elif isinstance(value, datetime):
                date_value = value
            else:
                return False
                
            
            params = list(params)# Parse comparison date
            params[0] = self.get_field_value(params[0], params[0])
            compare_date = parse(params[0])
            
            return date_value > compare_date
            
        except (ValueError, TypeError):
            return False
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must be after :value"
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':value': self.get_field_value(self._params[0], self._params[0])
        }
        
        return replacements
    
class AfterOrEqualRule(Rule):
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        try:
            if isinstance(value, str):
                date_value = parse(value)
            elif isinstance(value, datetime):
                date_value = value
            else:
                return False
            
            params = list(params)
            params[0] = self.get_field_value(params[0], params[0])
            compare_date = parse(params[0])
            
            return date_value >= compare_date
            
        except (ValueError, TypeError) as e:
            return False
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must be after or equal to :value"
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':value': self.get_field_value(self._params[0], self._params[0])
        }
        
        return replacements
    
class BeforeRule(Rule):
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        value = self.get_field_value(value, value)
    
        try:
            if isinstance(value, str):
                date_value = parse(value)
            elif isinstance(value, datetime):
                date_value = value
            else:
                return False
            
            params = list(params)
            params[0] = self.get_field_value(params[0], params[0])
            compare_date = parse(params[0])
            
            return date_value < compare_date
            
        except (ValueError, TypeError):
            return False
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must be before :value"
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':value': self.get_field_value(self._params[0], self._params[0])
        }
        
        return replacements
    
class BeforeOrEqualRule(Rule):
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        value = self.get_field_value(value, value) 
        
        try:
            if isinstance(value, str):
                date_value = parse(value)
            elif isinstance(value, datetime):
                date_value = value
            else:
                return False
            
            params = list(params)
            params[0] = self.get_field_value(params[0], params[0])
            compare_date = parse(params[0])
            
            return date_value <= compare_date
            
        except (ValueError, TypeError):
            return False
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must be before or equal to :value"
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':value': self.get_field_value(self._params[0], self._params[0])
        }
        
        return replacements
    
class DateFormatRule(Rule):
    _count_parameter = 1
    
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        try:
            datetime.strptime(value, params[0])
            return True
        except ValueError:
            return False
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must match the format :format"
    
    def replacements(self, field, value):
        replacements = {
            ':attribute': self._get_display_name(field),
            ':input': value,
            ':format': self.get_field_value(self._params[0], self._params[0])
        }
        
        return replacements
    
class TimezoneRule(Rule):
    def validate(self, field: str, value: Any, params: List[str]) -> bool:
        try:
            zoneinfo.ZoneInfo(value)
            return True
        except zoneinfo.ZoneInfoNotFoundError:
            return False
    
    def message(self, field: str, params: List[str]) -> str:
        return f"The :attribute must be a valid timezone."